@@ -35,6 +35,7 @@
 #include <libanjuta/interfaces/ianjuta-editor-zoom.h>
 #include <libanjuta/interfaces/ianjuta-editor-goto.h>
 #include <libanjuta/interfaces/ianjuta-editor-language.h>
+#include <libanjuta/interfaces/ianjuta-editor-assist.h>
 
 #include <sys/wait.h>
 #include <sys/stat.h>
@@ -81,7 +82,7 @@ get_current_popup_active (gpointer user_data)
 	if (widget)
 	{
 		widget = gtk_widget_get_toplevel (widget);
-		if (GTK_WIDGET_TOPLEVEL (widget))
+		if (gtk_widget_is_toplevel (widget))
 			return gtk_window_has_toplevel_focus (GTK_WINDOW (widget));
 	}
 	return FALSE;
@@ -256,6 +257,43 @@ on_close_all_file_activate (GtkAction *action, gpointer user_data)
 	}
 }
 
+void 
+on_close_other_file_activate (GtkAction *action, gpointer user_data)
+{
+	GList *buffers;
+	AnjutaDocman *docman;
+	IAnjutaDocument *curr_doc;
+	DocmanPlugin *plugin;
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	docman = ANJUTA_DOCMAN (plugin->docman);
+	
+	/* Close all 'saved' files */
+	buffers = anjuta_docman_get_all_doc_widgets (docman);
+	if (buffers)
+	{
+		GList *node;
+		node = buffers;
+
+		curr_doc = anjuta_docman_get_current_document (docman);
+		while (node)
+		{	
+			IAnjutaDocument *doc;
+			GList* next;
+			doc = IANJUTA_DOCUMENT (node->data);
+			next = g_list_next (node); /* grab it now, as we may change it. */
+			if (doc && doc != curr_doc)
+			{
+				if (!ianjuta_file_savable_is_dirty (IANJUTA_FILE_SAVABLE (doc), NULL))
+				{
+					anjuta_docman_remove_document (docman, doc);
+				}
+			}
+			node = next;
+		}
+		g_list_free (buffers);
+	}	
+}
+
 void
 on_reload_file_activate (GtkAction *action, gpointer user_data)
 {
@@ -695,13 +733,13 @@ on_goto_block_end1_activate (GtkAction *action, gpointer user_data)
 		ianjuta_editor_goto_end_block (IANJUTA_EDITOR_GOTO (doc), NULL);
 }
 
-#define VIEW_LINENUMBERS_MARGIN    "margin.linenumber.visible"
-#define VIEW_MARKER_MARGIN         "margin.marker.visible"
-#define VIEW_FOLD_MARGIN           "margin.fold.visible"
-#define VIEW_INDENTATION_GUIDES    "view.indentation.guides"
-#define VIEW_WHITE_SPACES          "view.whitespace"
-#define VIEW_EOL                   "view.eol"
-#define VIEW_LINE_WRAP             "view.line.wrap"
+#define VIEW_LINENUMBERS_MARGIN    "docman-linenumber-visible"
+#define VIEW_MARKER_MARGIN         "docman-marker-visible"
+#define VIEW_FOLD_MARGIN           "docman-fold-visible"
+#define VIEW_INDENTATION_GUIDES    "docman-indentation-guides"
+#define VIEW_WHITE_SPACES          "docman-whitespace"
+#define VIEW_EOL                   "docman-eol"
+#define VIEW_LINE_WRAP             "docman-line-wrap"
 
 void
 on_editor_linenos1_activate (GtkAction *action, gpointer user_data)
@@ -710,8 +748,8 @@ on_editor_linenos1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_bool (plugin->prefs,
-								VIEW_LINENUMBERS_MARGIN, state);
+	g_settings_set_boolean (plugin->settings,
+	                        VIEW_LINENUMBERS_MARGIN, state);
 }
 
 void
@@ -721,8 +759,8 @@ on_editor_markers1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_bool (plugin->prefs,
-								VIEW_MARKER_MARGIN, state);
+	g_settings_set_boolean (plugin->settings,
+	                        VIEW_MARKER_MARGIN, state);
 }
 
 void
@@ -732,8 +770,8 @@ on_editor_codefold1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_bool (plugin->prefs,
-								VIEW_FOLD_MARGIN, state);
+	g_settings_set_boolean (plugin->settings,
+	                        VIEW_FOLD_MARGIN, state);
 }
 
 void
@@ -743,8 +781,8 @@ on_editor_indentguides1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_bool (plugin->prefs,
-								VIEW_INDENTATION_GUIDES, state);
+	g_settings_set_boolean (plugin->settings,
+	                        VIEW_INDENTATION_GUIDES, state);
 }
 
 void
@@ -754,8 +792,8 @@ on_editor_whitespaces1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_bool (plugin->prefs,
-								VIEW_WHITE_SPACES, state);
+	g_settings_set_boolean (plugin->settings,
+	                        VIEW_WHITE_SPACES, state);
 }
 
 void
@@ -765,8 +803,8 @@ on_editor_eolchars1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_bool (plugin->prefs,
-								VIEW_EOL, state);
+	g_settings_set_boolean (plugin->settings,
+	                        VIEW_EOL, state);
 }
 
 void
@@ -776,8 +814,8 @@ on_editor_linewrap1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_bool (plugin->prefs,
-								VIEW_LINE_WRAP, state);
+	g_settings_set_boolean (plugin->settings,
+	                        VIEW_LINE_WRAP, state);
 }
 
 void
@@ -828,7 +866,7 @@ on_show_search (GtkAction *action, gpointer user_data)
 	if (!gtk_widget_get_parent (search_box))
 		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
 
-	search_box_fill_search_focus (SEARCH_BOX (search_box));
+	search_box_fill_search_focus (SEARCH_BOX (search_box), FALSE);
 	gtk_widget_show (search_box);
 }
 
@@ -844,9 +882,99 @@ on_repeat_quicksearch (GtkAction *action, gpointer user_data)
 	if (!gtk_widget_get_parent (search_box))
 		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
 
-	if (!GTK_WIDGET_VISIBLE (search_box))
+	if (!gtk_widget_get_visible (search_box))
 		gtk_widget_show (search_box);
-	on_search_activated (NULL, SEARCH_BOX (search_box));
+	search_box_incremental_search (SEARCH_BOX (search_box), TRUE);
+}
+
+void
+on_search_previous (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	GtkWidget *search_box;
+
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+
+	search_box = plugin->search_box;
+	if (!gtk_widget_get_parent (search_box))
+		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
+
+	if (!gtk_widget_get_visible (search_box))
+		gtk_widget_show (search_box);
+	search_box_incremental_search (SEARCH_BOX (search_box), FALSE);
+}
+
+void 
+on_search_and_replace (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	GtkWidget *search_box;
+
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	
+	search_box = plugin->search_box;
+	if (!gtk_widget_get_parent (search_box))
+		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
+
+	search_box_fill_search_focus (SEARCH_BOX (search_box), TRUE);
+	gtk_widget_show (search_box);
+}
+
+void 
+on_search_popup_case_sensitive_toggle (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	GtkWidget *search_box;
+
+	gboolean case_sensitive_status = 
+		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+
+	search_box = plugin->search_box;
+	if (!gtk_widget_get_parent (search_box))
+		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
+
+	if (!gtk_widget_get_visible (search_box))
+		gtk_widget_show (search_box);
+	
+	search_box_toggle_case_sensitive (SEARCH_BOX (search_box), case_sensitive_status);
+}
+
+void 
+on_search_popup_highlight_toggle (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	GtkWidget *search_box;
+
+	gboolean highlight_status = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));	
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+
+	search_box = plugin->search_box;
+	if (!gtk_widget_get_parent (search_box))
+		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
+
+	if (!gtk_widget_get_visible (search_box))
+		gtk_widget_show (search_box);
+
+	search_box_toggle_highlight (SEARCH_BOX (search_box), highlight_status);
+}
+
+void
+on_search_popup_clear_highlight (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	GtkWidget *search_box;
+
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+
+	search_box = plugin->search_box;
+	if (!gtk_widget_get_parent (search_box))
+		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
+
+	if (!gtk_widget_get_visible (search_box))
+		gtk_widget_show (search_box);
+
+	search_box_clear_highlight (SEARCH_BOX (search_box));
 }
 
 void
@@ -905,7 +1033,7 @@ on_previous_document (GtkAction *action, gpointer user_data)
 }
 
 void
-on_bookmark_add_activate (GtkAction *action, gpointer user_data)
+on_bookmark_toggle_activate (GtkAction *action, gpointer user_data)
 {
 	IAnjutaDocument *doc;
 	DocmanPlugin *plugin;
@@ -914,8 +1042,8 @@ on_bookmark_add_activate (GtkAction *action, gpointer user_data)
 	if (doc && IANJUTA_IS_EDITOR(doc))
 	{
 		IAnjutaEditor* editor = IANJUTA_EDITOR(doc);
-		anjuta_bookmarks_add (ANJUTA_BOOKMARKS (plugin->bookmarks), editor, 
-							  ianjuta_editor_get_lineno (editor, NULL), NULL, TRUE);
+		anjuta_bookmarks_toggle (ANJUTA_BOOKMARKS (plugin->bookmarks), editor, 
+							  ianjuta_editor_get_lineno (editor, NULL));
 	}
 }
 
@@ -956,3 +1084,17 @@ on_bookmarks_clear_activate (GtkAction *action, gpointer user_data)
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	anjuta_bookmarks_clear (ANJUTA_BOOKMARKS(plugin->bookmarks));
 }
+
+void
+on_autocomplete_activate (GtkAction *action, gpointer user_data)
+{
+	IAnjutaDocument *doc;
+	DocmanPlugin *plugin;
+	doc = get_current_document (user_data);
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	if (doc && IANJUTA_IS_EDITOR_ASSIST(doc))
+	{
+		IAnjutaEditorAssist* assist = IANJUTA_EDITOR_ASSIST (doc);
+		ianjuta_editor_assist_invoke (assist, NULL, NULL);
+	}   
+}
