@@ -49,6 +49,16 @@ struct _AnjutaFileViewPrivate
 	GtkTreeRowReference* current_selection;
 };
 
+GtkTargetEntry uri_targets[] =
+{
+	{
+		"text/uri-list",
+		0, /* no flags */
+		0
+	}
+};
+	
+
 #define ANJUTA_FILE_VIEW_GET_PRIVATE(o) \
 	(G_TYPE_INSTANCE_GET_PRIVATE((o), ANJUTA_TYPE_FILE_VIEW, AnjutaFileViewPrivate))
 
@@ -79,9 +89,13 @@ get_status_string(AnjutaVcsStatus status)
 			return _("Unversioned");
 		case ANJUTA_VCS_STATUS_IGNORED:
 			return _("Ignored");
+#if 0
 		default:
 			g_assert_not_reached();
+#endif
 	}		
+
+	return NULL;
 }
 
 enum
@@ -123,13 +137,40 @@ file_view_get_selected (AnjutaFileView* view)
 	GtkTreeIter selected;
 	if (gtk_tree_selection_get_selected (selection, NULL, &selected))
 	{
-		GFile* file = file_model_get_file (priv->model, &selected);
+		GFile* file;
+		GtkTreeIter real_iter;
+		GtkTreeModel* sort_model = gtk_tree_view_get_model (GTK_TREE_VIEW(view));
+
+		gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT(sort_model),
+		                                               &real_iter, &selected);
+
+		file = file_model_get_file (priv->model, &real_iter);
 		return file;
 	}
 	else
 		return NULL;
 }
 
+static void
+file_view_drag_data_get (GtkWidget* widget,
+                         GdkDragContext *drag_context,
+                         GtkSelectionData *data,
+                         guint info,
+                         guint time)
+{
+	AnjutaFileView* view = ANJUTA_FILE_VIEW(widget);
+	GFile* selected = file_view_get_selected (view);
+
+	if (selected)
+	{
+		gchar* uris[2];
+		uris[0] = g_file_get_uri (selected);
+		uris[1] = NULL;
+		gtk_selection_data_set_uris (data, uris);
+		g_free(uris[0]);
+	}
+}
+
 static void
 file_view_row_activated (GtkTreeView* widget, GtkTreePath* sort_path,
 						 GtkTreeViewColumn* column)
@@ -183,7 +224,7 @@ file_view_row_activated (GtkTreeView* widget, GtkTreePath* sort_path,
 static gboolean
 file_view_key_press_event (GtkWidget* widget, GdkEventKey* event)
 {
-	if (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter)
+	if (event->keyval == GDK_KEY_Return || event->keyval == GDK_KEY_KP_Enter)
 	{
 		AnjutaFileView* view = ANJUTA_FILE_VIEW (widget);
 		AnjutaFileViewPrivate* priv = ANJUTA_FILE_VIEW_GET_PRIVATE (view);
@@ -258,16 +299,38 @@ file_view_do_popup_menu (GtkWidget* widget, GdkEventButton* event)
 static gboolean
 file_view_button_press_event (GtkWidget *widget, GdkEventButton *event)
 {
-  /* Ignore double-clicks and triple-clicks */
-  if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
-    {
-      file_view_do_popup_menu (widget, event);
-      return TRUE;
-    }
-
- return 	
+	gint retval = 
 		GTK_WIDGET_CLASS (file_view_parent_class)->button_press_event (widget,
 																	   event);
+	/* Ignore double-clicks and triple-clicks */
+	if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
+    {
+		GtkTreePath *path;
+
+		/* Select file under cursor if not selected */
+        if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget),
+                        event->x,event->y, &path, NULL, NULL, NULL))
+		{
+			GtkTreeSelection *selection;
+
+			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (widget));
+	        if (!gtk_tree_selection_path_is_selected(selection, path))
+    	    {
+        	    gtk_tree_selection_unselect_all(selection);
+	            gtk_tree_selection_select_path(selection, path);
+    	        gtk_tree_view_set_cursor(GTK_TREE_VIEW(widget),
+        	                        path, NULL, FALSE);
+        	}
+        	gtk_tree_path_free (path);
+			
+			file_view_do_popup_menu (widget, event);
+		
+      		return TRUE;
+		}
+    }
+
+	return 	retval;
+		
 }
 
 static gboolean
@@ -391,6 +454,12 @@ file_view_query_tooltip (GtkWidget* widget, gint x, gint y, gboolean keyboard,
 	GtkTreeIter iter;
 	GtkTreeIter real_iter;
 	gchar* filename;
+	gboolean result = TRUE;
+	GdkRectangle visible_rect, column_rect;
+	GdkScreen *screen = gdk_screen_get_default ();
+	PangoContext *context;
+	PangoLayout *layout;
+	gint width, height;
 	
 	if (!gtk_tree_view_get_tooltip_context (GTK_TREE_VIEW (view),
 											&x, &y, keyboard,
@@ -403,15 +472,35 @@ file_view_query_tooltip (GtkWidget* widget, gint x, gint y, gboolean keyboard,
 											&real_iter, &iter);
 	
 	filename = file_model_get_filename (FILE_MODEL (file_model), &real_iter);
-	gtk_tooltip_set_text (tooltip, filename);
-	gtk_tree_view_set_tooltip_row (GTK_TREE_VIEW (view),
-								   tooltip,
-								   path);
-	
+
+	context = gdk_pango_context_get_for_screen (screen);
+	layout = pango_layout_new (context);
+
+	pango_layout_set_text (layout, filename, -1);
+	pango_layout_get_pixel_size (layout, &width, &height);
+
+	gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (view), &visible_rect);
+	gtk_tree_view_get_cell_area (GTK_TREE_VIEW (view), path,
+								 gtk_tree_view_get_column (GTK_TREE_VIEW (view), 0), &column_rect);
+
+	if (column_rect.x + width > visible_rect.x + visible_rect.width ||
+		column_rect.x < visible_rect.x)
+	{
+		gtk_tooltip_set_text (tooltip, filename);
+		gtk_tree_view_set_tooltip_row (GTK_TREE_VIEW (view),
+									   tooltip,
+									   path);
+	} else
+	{
+		result = FALSE;
+	}
+
 	g_free (filename);
 	gtk_tree_path_free (path);
+	g_object_unref (layout);
+	g_object_unref (context);
 	
-	return TRUE;
+	return result;
 }
 
 static int
@@ -444,7 +533,7 @@ file_view_sort_model(GtkTreeModel* model,
 	}
 	else if (filename1 && filename2)
 	{
-		retval = strcmp(filename1, filename2);
+		retval = strcasecmp(filename1, filename2);
 	}
 	g_free(filename1);
 	g_free(filename2);
@@ -496,6 +585,13 @@ file_view_init (AnjutaFileView *object)
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (object), FALSE);
 	
 	g_object_set (object, "has-tooltip", TRUE, NULL);
+
+	/* DND */
+	gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW(object),
+	                                        GDK_BUTTON1_MASK,
+	                                        uri_targets,
+	                                        1,
+	                                        GDK_ACTION_MOVE);                                    
 }
 
 static void
@@ -557,8 +653,8 @@ file_view_class_init (AnjutaFileViewClass *klass)
 	g_object_class_install_property (object_class,
 									 PROP_BASE_URI,
 									 g_param_spec_string ("base_uri",
-														  _("Base uri"),
-														  _("Uri of the top-most path displayed"),
+														  _("Base URI"),
+														  _("URI of the top-most path displayed"),
 														  NULL,
 														  G_PARAM_READABLE |
 														  G_PARAM_WRITABLE |
@@ -605,7 +701,10 @@ file_view_class_init (AnjutaFileViewClass *klass)
 	widget_class->button_press_event = file_view_button_press_event;
 	
 	/* Tooltips */
-	widget_class->query_tooltip = file_view_query_tooltip;	
+	widget_class->query_tooltip = file_view_query_tooltip;
+
+	/* Dnd */
+	widget_class->drag_data_get = file_view_drag_data_get;
 }
 
 GtkWidget*
